import { MD5 } from 'crypto-js';

type CodeUrl = {
  url: string; // 地址
  code?: string; // 代码
  id?: string; // codeId
  hash?: string;
};
export class CodeUrlManager {
  urls: CodeUrl[] = [];
  urlMap: { [key: string]: CodeUrl } = {};
  constructor() {
    this.urls = [];
  }
  addCode(code: string, id: string, hash?: string) {
    const urlMap = this.urlMap;
    const resultContent = (content: CodeUrl) => {
      const index = this.urls.findIndex((item) => item.id === id);
      if (index === -1) {
        this.urls.push(content);
      }
      return content;
    };
    if (urlMap[hash]) {
      // 可能id不同，但是hash相同。已经加载过了。
      const content = {
        ...urlMap[hash],
        id,
      };
      return resultContent(content);
    }
    const _hash = hash || MD5(code).toString();
    if (urlMap[_hash]) {
      const content = {
        ...urlMap[_hash],
        id,
      };
      return resultContent(content);
    }
    const index = this.urls.findIndex((item) => item.id === id);
    if (index > -1) {
      // 没有共同hash的值了，但是还是找了id，这个时候应该移除之前
      const url = this.urls[index];
      const list = this.urls.filter((item) => item.hash === url.hash);
      if (list.length <= 1) {
        // 这个hash的值没有人用了
        URL.revokeObjectURL(url.url);
        this.urlMap[url.hash] = null;
      }
      this.urls.splice(index, 1);
    }
    /**
     * 全新代码
     *
     */
    const url = URL.createObjectURL(new Blob([code], { type: 'application/javascript' }));
    const content = {
      id,
      url,
      code,
      hash: _hash,
    };

    this.urls.push(content);
    urlMap[_hash] = { ...content, id: null };
    return content;
  }
  removeUrl(id?: string, url?: string) {
    const index = this.urls.findIndex((item) => item.url === url || item.id === id);
    if (index > -1) {
      const url = this.urls[index];
      URL.revokeObjectURL(url.url);
      this.urlMap[url.hash] = null;
      this.urls.splice(index, 1);
    }
  }
}
export const codeUrlManager = new CodeUrlManager();
