import type { CodeUrlManager } from '../code-url';
type Fn = (args: any) => any;
export type RenderCode = {
  render: Fn;
  unmount?: Fn;
  [key: string]: any;
};

type AdapterData = {
  codeId?: string;
  code?: string;
  hash?: string;
  codeUrl?: string;
};
type CodeAdapterOpts<T = any> = {
  handle?: any;
} & T;
export class CodeAdapter {
  handle: (data: AdapterData) => Promise<any>;
  constructor(opts: CodeAdapterOpts) {
    this.handle = opts.handle;
  }
  setHandle(handle: (data: AdapterData) => Promise<any>) {
    this.handle = handle;
  }
}
type Opts = {
  codeUrlManager: CodeUrlManager;
};
export class CodeUrlAdapter extends CodeAdapter {
  codeUrlManager: CodeUrlManager;
  constructor(opts: CodeAdapterOpts<Opts>) {
    super(opts);
    this.handle = (data) => handleOneCode(data, this.codeUrlManager);
    this.codeUrlManager = opts.codeUrlManager;
  }
  setCodeUrlManager(codeUrlManager: CodeUrlManager) {
    this.codeUrlManager = codeUrlManager;
  }
}
export const handleOneCode = async (data: AdapterData, codeUrlManager: CodeUrlManager) => {
  const { code, codeId } = data;
  const urlsBegin = ['http', 'https', 'blob'];

  let importUrl = '';
  // codeUrl优先级最高
  if (data?.codeUrl) {
    importUrl = data.codeUrl;
  } else if (typeof code === 'string' && code && urlsBegin.find((item) => code.startsWith(item))) {
    importUrl = code;
  }
  if (importUrl) {
    const importContent = await import(/* @vite-ignore */ importUrl);
    const { render, unmount, ...rest } = importContent || {};
    const _data = {
      ...data,
      code: {
        render,
        unmount,
        ...rest,
      },
    };
    return _data;
  }
  if (typeof code === 'string' && codeId) {
    const { url, hash } = codeUrlManager.addCode(code, codeId, data.hash);
    const importContent = await import(/* @vite-ignore */ url);
    const { render, unmount, ...rest } = importContent || {};
    const _data = {
      ...data,
      codeId,
      hash,
      code: {
        render,
        unmount,
        ...rest,
      },
    };
    return _data;
  } else if (typeof code === 'string' && !codeId) {
    console.error('codeId is required', data);
  }
  // 无需处理
  return data;
};
