// 定义任务队列
const highPriorityTasks = [];
const lowPriorityTasks = [];
let isSchedulerRunning = false; // 调度器状态

export const fnvoid = () => {};
export type Priority = 'high' | 'low';
// 添加任务的辅助函数
export const addTask = (callback: any, priority: Priority = 'low') => {
  if (priority === 'high') {
    highPriorityTasks.push(callback);
  } else {
    lowPriorityTasks.push(callback);
  }
  // 如果调度器未运行且有任务，启动调度
  if (!isSchedulerRunning) {
    isSchedulerRunning = true;
    requestIdleCallback(processTasks);
  }
};

// 调度函数
const processTasks: IdleRequestCallback = (deadline) => {
  // 优先处理高优先级任务
  while (highPriorityTasks.length && deadline.timeRemaining() > 0) {
    const task = highPriorityTasks.shift();
    task();
  }

  // 如果有剩余时间，再处理低优先级任务
  while (lowPriorityTasks.length && deadline.timeRemaining() > 0) {
    const task = lowPriorityTasks.shift();
    task();
  }

  // 检查是否还有任务
  if (highPriorityTasks.length || lowPriorityTasks.length) {
    requestIdleCallback(processTasks);
  } else {
    // 没有任务，停止调度器
    isSchedulerRunning = false;
  }
};
// 开始调度
requestIdleCallback(processTasks);

// 添加一些测试任务
// addTask(() => console.log('Low priority task 1'), 'low');
// addTask(() => console.log('High priority task 1'), 'high');
// addTask(() => console.log('Low priority task 2'), 'low');
// addTask(() => console.log('High priority task 2'), 'high');

export const getLoading = () => {
  return isSchedulerRunning;
};
